"use strict";
module.exports = Builder;
var Query = require('./query.js');

/**
 * Query builder
 * @constructor
 * @implements {Query}
 */
function Builder() {
	Query.apply(this, arguments);
}

Builder.inherits(Query);

/**
 * Compiles an array of JOIN statements into an SQL partial.
 *
 * @protected
 * @param   {Join[]}   join statements
 * @return  {string}
 */
Builder.prototype._compileJoin = function _compileJoin(joins) {
	return joins.map(function (join) {
		// Compile each of the join statements
		return join.compile();
	}, this).join(' ');
};

/**
 * Compiles an array of conditions into an SQL partial. Used for WHERE
 * and HAVING.
 *
 * @protected
 * @param   {Array} conditions  condition statements
 * @this {Builder}
 * @return  {string}
 */
Builder.prototype._compileСonditions = function _compileСonditions(conditions) {
	var lastCondition = null, sql = '', db = this.db;
	conditions.forEach(function (group) {
		var logic;
		// Process groups of conditions
		for (logic in group) {
			var condition = group[logic];
			if (typeof condition == 'string' && condition == '(') {
				if (sql && lastCondition != '(') {
					// Include logic operator
					sql += ' ' + logic + ' ';
				}

				sql += '(';
			} else if (typeof condition == 'string' && condition == ')') {
				sql += ')';
			} else {
				if (sql && ((lastCondition instanceof Array) || (lastCondition != '('))) {
					// Add the logic operator
					sql += ' ' + logic + ' ';
				}

				// Split the condition
				var column = condition[0],
					op = condition[1],
					value = condition[2];

				if (value === null) {
					if (op === '=') {
						// Convert "val = NULL" to "val IS NULL"
						op = 'IS';
					} else if (op === '!=') {
						// Convert "val != NULL" to "valu IS NOT NULL"
						op = 'IS NOT';
					}
				}

				// Database operators are always uppercase
				op = op.toUpperCase();

				if (op === 'BETWEEN' && Array.isArray(value)) {
					// BETWEEN always has exactly two arguments
					var min = value[0],
						max = value[1];

					if ((typeof min == 'string') && (!(min in this._params))) {
						// Quote the value, it is not a parameter
						min = this.db.quote(min);
					}

					if ((typeof max == 'string') && (!(max in this._params))) {
						// Quote the value, it is not a parameter
						max = this.db.quote(max);
					}

					// Quote the min and max value
					value = min + ' AND ' + max;
				} else if ((typeof value != 'string') || (!(value in this._params))) {
					// Quote the value, it is not a parameter
					value = this.db.quote(value);
				}

				if (column) {
					if (Array.isArray(column)) {
						// Use the column name
						column = this.db.quoteIdentifier(column);
					} else {
						// Apply proper quoting to the column
						column = this.db.quoteColumn(column);
					}
				}

				// Append the statement to the query
				sql += column + ' ' + op + ' ' + value;
			}

			lastCondition = condition;
		}
	}, this);
	return sql;
};

/**
 * Compiles an array of set values into an SQL partial. Used for UPDATE.
 *
 * @protected
 * @param   {Array}   updated values
 * @return  {string}
 */
Builder.prototype._compileSet = function _compileSet(values) {
	return values.map(function (group) {
		// Split the set
		var column = group[0],
			value = group[1];
		// Quote the column name
		column = this.db.quoteColumn(column);

		if ((typeof value != 'string') || (!(value in this._params))) {
			// Quote the value, it is not a parameter   
			value = this.db.quote(value);
		}
		return column + ' = ' + value;

	}, this.join(', '));
};

/**
 * Compiles an array of GROUP BY columns into an SQL partial.
 *
 * @protected
 * @param   {Array}   columns
 * @return  {string}
 */
Builder.prototype._compileGroupBy = function _compileGroupBy(columns) {
	return 'GROUP BY ' + columns.map(function (column) {
		if (Array.isArray(column)) {
			return this.db.quoteIdentifier(column.pop());
		} else {
			// Apply proper quoting to the column
			return this.db.quoteColumn(column);
		}
	}, this).join(', ');
};

/**
 * Compiles an array of ORDER BY statements into an SQL partial.
 *
 * @protected
 * @param   {Array}   sorting columns
 * @return  {string}
 */
Builder.prototype._compileOrderBy = function _compileOrderBy(columns) {
	return 'ORDER BY ' + columns.map(function (group) {
		var column = group[0];
		var direction = group[1];
		if (Array.isArray(column)) {
			// Use the column alias
			column = this.db.quoteIdentifier(column.pop());
		} else {
			// Apply proper quoting to the column
			column = this.db.quoteColumn(column);
		}

		if (typeof direction == 'string') {
			// Make the direction uppercase
			column += ' ' + direction.toUpperCase();
		}

		return column;
	}, this).join(', ');
};

/**
 * Reset the current builder status.
 *
 * @return  {Builder}
 */
Builder.prototype.reset = function reset() {
	this._sql = null;
	return Query.prototype.reset.call(this);
};