"use strict";
module.exports = DB;
var Exception = require('./exception.js');
/**
 * Базовый прототип соединения с БД.
 */
function DB(core, config) {
	Object.defineProperties(this, {
		core: {
			value: core,
			writable: false},
		config: {
			value: config,
			writable: false},
		cache: {
			value: new (require('./cache.js')),
			writable: false}
	});
	this.connect();
}

var Query = require('./query.js'),
	Builder = require('./builder.js'),
	Select = require('./select.js'),
	Insert = require('./insert.js'),
	Expression = require('./expression.js');

/**
 * Select query type
 *
 * @const
 * @type {Number}
 */
DB.prototype.SELECT = 1;

/**
 * Insert query type
 *
 * @const
 * @type {Number}
 */
DB.prototype.INSERT = 2;

/**
 * Update query type
 *
 * @const
 * @type {Number}
 */
DB.prototype.UPDATE = 3;

/**
 * Delete query type
 *
 * @const
 * @type {Number}
 */
DB.prototype.DELETE = 4;

/**
 * Quoted char
 *
 * @type {String}
 * @private
 */
DB.prototype._identifier = '';

/**
 * Raw connection
 * @type {*}
 * @private
 */
DB.prototype._connection = null;

/**
 * Установка соединения с сервером
 */
DB.prototype.connect = function connect() {

};

/**
 * Отключение от сервера
 */
DB.prototype.disconnect = function disconnect() {

};

/**
 * Set the connection character set. This is called automatically by [DB.connect].
 *
 * `db->set_charset('utf8');`
 *
 * @param {string} charset
 */
DB.prototype.setCharset = function setCharset(charset) {
	throw new Exception(0, 'Abstract method');
};

/**
 * Perform an SQL query of the given type.
 *
 *     // Make a SELECT query and use objects for results
 *     core.db.query(core.db.SELECT, 'SELECT * FROM groups');
 *
 *     // Make a SELECT query and use "Model_User" for the results
 *     $db->query(DB::SELECT, 'SELECT * FROM users LIMIT 1', 'Model_User');
 *
 * @param   integer  DB::SELECT, DB::INSERT, etc
 * @param   string   SQL query
 * @param   mixed    result object class string, TRUE for stdClass, FALSE for assoc array
 * @param   array    object construct parameters for result class
 * @return  object   DB_Result for SELECT queries
 * @return  array    list (insert id, row count) for INSERT queries
 * @return  integer  number of affected rows for all other queries
 */
DB.prototype.query = function query(type, sql, toObject, callback) {
	callback(new Exception(0, 'Abstract method'));
};


/**
 * Count the number of records in a table.
 *
 *     // Get the total number of records in the "users" table
 *     $count = $db->count_records('users');
 *
 * @param   mixed    table name string or array(query, alias)
 * @return  integer
 */
DB.prototype.countRecords = function countRecords(table, callback) {
	callback(new Exception(0, 'Abstract method'));
};


/**
 * Returns a normalized array describing the SQL data type
 *
 *     $db->datatype('char');
 *
 * @param   string  SQL data type
 * @return  array
 */
DB.prototype.datatype = function datatype(type) {
	if (type in this.types) {
		return this.types[type];
	}
	return {};
};

/**
 * List all of the tables in the DB. Optionally, a LIKE string can
 * be used to search for specific tables.
 *
 *     // Get all tables in the current DB
 *     $tables = $db->list_tables();
 *
 *     // Get all user-related tables
 *     $tables = $db->list_tables('user%');
 *
 * @param   string   table to search for
 * @return  array
 */
DB.prototype.listTables = function listTables(like, callback) {
	callback(new Exception(0, 'Abstract method'));
};


/**
 * Lists all of the columns in a table. Optionally, a LIKE string can be
 * used to search for specific fields.
 *
 *     // Get all columns from the "users" table
 *     $columns = $db->list_columns('users');
 *
 *     // Get all name-related columns
 *     $columns = $db->list_columns('users', '%name%');
 *
 *     // Get the columns from a table that doesn't use the table prefix
 *     $columns = $db->list_columns('users', NULL, FALSE);
 *
 * @param   string  table to get columns from
 * @param   string  column to search for
 * @param   boolean whether to add the table prefix automatically or not
 * @return  array
 */
DB.prototype.listColumns = function listColumns(table, like, add_prefix, callback) {
	callback(new Exception(0, 'Abstract method'));
};

/**
 * Extracts the text between parentheses, if any.
 *
 *     // Returns: array('CHAR', '6')
 *     list($type, $length) = $db->_parse_type('CHAR(6)');
 *
 * @param   string
 * @return  array   list containing the type and length, if any
 */
DB.prototype._parseType = function _parseType(type) {
	var open = type.indexOf('(');
	if (open == -1) {
		// No length specified
		return [type, null];
	}

	// Closing parenthesis
	var close = type.indexOf(')', open);

	// Length without parentheses
	var length = type.substr(open + 1, close - 1 - open) - 0;

	// Type without the length
	type = type.substr(0, open) + type.substr(close + 1);

	return [type, length];
};

/**
 * Return the table prefix defined in the current configuration.
 *
 *     $prefix = $db->table_prefix();
 *
 * @return  string
 */
DB.prototype.tablePrefix = function tablePrefix() {
	return this.config['prefix'] || '';
};

/**
 * Quote a value for an SQL query.
 *
 *     db.quote(NULL);   // 'NULL'
 *     db.quote(10);     // 10
 *     db.quote('fred'); // 'fred'
 *
 * Objects passed to this function will be converted to strings.
 * [DB_Expression] objects will be compiled.
 * [DB_Query] objects will be compiled and converted to a sub-query.
 * All other objects will be converted using the `__toString` method.
 *
 * @param   mixed   any value to quote
 * @return  string
 * @uses    DB::escape
 */
DB.prototype.quote = function quote(value) {
	if ((value == null) || (typeof value == 'undefined')) {
		return 'NULL';
	} else if (value === true) {
		return "'1'";
	} else if (value === false) {
		return "'0'";
	} else if (Array.isArray(value)) {
		var nv = value.map(this.quote, this);
		return '(' + nv.join(', ') + ')';
	} else if (typeof value == 'number') {
		return value;
	} else if (typeof value == 'string') {
		return this.escape(value);
	}
	if (value instanceof Date) {
		return this.quote(new Date(value - value.getTimezoneOffset() * 60000).toISOString());
	}
	if (value instanceof Query) {
		// Create a sub-query
		return '(' + value.compile(this) + ')';
	} else if (value instanceof Expression) {
		// Compile the expression
		return value.compile(this);
	} else {
		// Convert the object to a string
		return this.quote(value.toString());
	}
};

/**
 * Quote a DB column name and add the table prefix if needed.
 *
 *     $column = $db->quote_column($column);
 *
 * You can also use SQL methods within identifiers.
 *
 *     // The value of "column" will be quoted
 *     $column = $db->quote_column('COUNT("column")');
 *
 * Objects passed to this function will be converted to strings.
 * [DB_Expression] objects will be compiled.
 * [DB_Query] objects will be compiled and converted to a sub-query.
 * All other objects will be converted using the `__toString` method.
 *
 * @param   mixed   column name or array(column, alias)
 * @return  string
 * @uses    DB::quote$identifier
 * @uses    DB::table_prefix
 */
DB.prototype.quoteColumn = function quoteColumn(column) {
	var alias = false;
	if (Array.isArray(column)) {
		alias = column[1];
		column = column[0];
	}

	if (column instanceof Query) {
		// Create a sub-query
		column = '(' + column.compile(this) + ')';
	} else if (column instanceof Expression) {
		// Compile the expression
		column = column.compile(this);
	} else {
		// Convert to a string
		column = new String(column);

		if (column == '*') {
			return column;
		} else if (column.indexOf('"') >= 0) {
			// Quote the column in FUNC("column") identifiers
			column = column.replace(/^"(.+?)"$/, this.quoteColumn.bind(this));
		} else if (column.indexOf('.') >= 0) {
			var parts = column.split('.');
			var prefix = this.tablePrefix();
			if (prefix) {
				// Get the offset of the table name, 2nd-to-last part
				var offset = parts.length - 2;

				// Add the table prefix to the table name
				parts[offset] = prefix + parts[offset];
			}
			parts.forEach(function (part, key) {
				if (part != '*') {
					// Quote each of the parts
					parts[key] = this._identifier + part + this._identifier;
				}
			}, this);

			column = parts.join('.');
		} else {
			column = this._identifier + column + this._identifier;
		}
	}

	if (alias) {
		column += ' AS ' + this._identifier + alias + this._identifier;
	}

	return column;
};

/**
 * Quote a DB table name and adds the table prefix if needed.
 *
 *     $table = $db->quote_table($table);
 *
 * Objects passed to this function will be converted to strings.
 * [DB_Expression] objects will be compiled.
 * [DB_Query] objects will be compiled and converted to a sub-query.
 * All other objects will be converted using the `__toString` method.
 *
 * @param   mixed   table name or array(table, alias)
 * @return  string
 * @uses    DB::quote$identifier
 * @uses    DB::table_prefix
 */
DB.prototype.quoteTable = function quoteTable(table) {
	var alias = false;
	if (Array.isArray(table)) {
		alias = table[1];
		table = table[0];
	}

	if (table instanceof Query) {
		// Create a sub-query
		table = '(' + table.compile(this) + ')';
	}
	else if (table instanceof Expression) {
		// Compile the expression
		table = table.compile(this);
	} else {
		// Convert to a string
		table = new String(table);

		if (table.indexOf('.') >= 0) {
			var parts = table.split('.');
			var prefix = this.tablePrefix();
			if (prefix) {
				// Get the offset of the table name, last part
				var offset = parts.length - 1;

				// Add the table prefix to the table name
				parts[offset] = prefix + parts[offset];
			}
			parts.forEach(function (part, key) {
				// Quote each of the parts
				parts[key] = this._identifier + part + this._identifier;
			}, this);

			table = parts.join('.');
		} else {
			// Add the table prefix
			table = this._identifier + this.tablePrefix() + table + this._identifier;
		}
	}

	if (alias) {
		// Attach table prefix to alias
		table += ' AS ' + this._identifier + this.tablePrefix() + alias + this._identifier;
	}

	return table;
};

/**
 * Quote a DB identifier
 *
 * Objects passed to this function will be converted to strings.
 * [DB_Expression] objects will be compiled.
 * [DB_Query] objects will be compiled and converted to a sub-query.
 * All other objects will be converted using the `__toString` method.
 *
 * @param   mixed   any identifier
 * @return  string
 */
DB.prototype.quoteIdentifier = function quoteIdentifier(value) {
	var alias = false;
	if (Array.isArray(value)) {
		alias = value[1];
		value = value[0];
	}

	if (value instanceof Query) {
		// Create a sub-query
		value = '(' + value.compile(this) + ')';
	} else if (value instanceof DB.Expression) {
		// Compile the expression
		value = value.compile(this);
	} else {
		// Convert to a string
		value = new String(value);

		if (value.indexOf('.') >= 0) {
			var parts = value.split('.');

			parts.forEach(function (part, key) {
				// Quote each of the parts
				parts[key] = this._identifier + part + this._identifier;
			}, this);

			value = parts.join('.');
		} else {
			value = this._identifier + value + this._identifier;
		}
	}

	if (alias) {
		value += ' AS ' + this._identifier + alias + this._identifier;
	}

	return value;
};

/**
 * Sanitize a string by escaping characters that could cause an SQL
 * injection attack.
 *
 *     $value = $db->escape('any string');
 *
 * @param   string   value to quote
 * @return  string
 */
DB.prototype.escape = function escape(value) {

};


//DB.Class = Class;


/**
 * Create a new [Database_Query] of the given type.
 *
 *     // Create a new SELECT query
 *     $query = DB::query(Database::SELECT, 'SELECT * FROM users');
 *
 *     // Create a new DELETE query
 *     $query = DB::query(Database::DELETE, 'DELETE FROM users WHERE id = 5');
 *
 * Specifying the type changes the returned result. When using
 * `Database::SELECT`, a [Database_Query_Result] will be returned.
 * `Database::INSERT` queries will return the insert id and number of rows.
 * For all other queries, the number of affected rows is returned.
 *
 * @param   integer  type: Database::SELECT, Database::UPDATE, etc
 * @param   string   SQL statement
 * @return  Database_Query
 */
DB.prototype.Query = function (type, sql) {
	return new Query(this, type, sql);
};

/**
 * Create a new [Database_Query_Builder_Select]. Each argument will be
 * treated as a column. To generate a `foo AS bar` alias, use an array.
 *
 *     // SELECT id, username
 *     $query = DB::select('id', 'username');
 *
 *     // SELECT id AS user_id
 *     $query = DB::select(array('id', 'user_id'));
 *
 * @param   mixed   column name or array($column, $alias) or object
 * @param   ...
 * @return  Database_Query_Builder_Select
 */
DB.prototype.Select = function () {
	return new Select(this, arguments);
};

/**
 * Create a new [Database_Query_Builder_Select] from an array of columns.
 *
 *     // SELECT id, username
 *     $query = DB::select_array(array('id', 'username'));
 *
 * @param   array   columns to select
 * @return  Database_Query_Builder_Select
 */
DB.prototype.SelectArray = function (columns) {
	return new Select(this, columns);
};

/**
 * Create a new [Database_Query_Builder_Insert].
 *
 *     // INSERT INTO users (id, username)
 *     $query = DB::insert('users', array('id', 'username'));
 *
 * @param   string  table to insert into
 * @param   array   list of column names or array($column, $alias) or object
 * @return  Database_Query_Builder_Insert
 */
DB.prototype.Insert = function (table, columns) {
	return new Insert(this, table, columns);
};

/**
 * Create a new [Database_Query_Builder_Update].
 *
 *     // UPDATE users
 *     $query = DB::update('users');
 *
 * @param   string  table to update
 * @return  Database_Query_Builder_Update
 */
DB.prototype.Update = function (table) {
	return new Builder.Update(this, table);
};

/**
 * Create a new [Database_Query_Builder_Delete].
 *
 *     // DELETE FROM users
 *     $query = DB::delete('users');
 *
 * @param   string  table to delete from
 * @return  Database_Query_Builder_Delete
 */
DB.prototype.Delete = function (table) {
	return new Builder.Delete(this, table);
};

/**
 * Create a new [Database_Expression] which is not escaped. An expression
 * is the only way to use SQL functions within query builders.
 *
 *     $expression = DB::expr('COUNT(users.id)');
 *     $query = DB::update('users')->set(array('login_count' => DB::expr('login_count + 1')))->where('id', '=', $id);
 *     $users = ORM::factory('user')->where(DB::expr("BINARY `hash`"), '=', $hash)->find();
 *
 * @param   string  expression
 * @param   array   parameters
 * @return  Database_Expression
 */
DB.prototype.Expr = function (string, parameters) {
	return new Expression(this, string, parameters);
};


DB.prototype.free = function(core) {
	this.disconnect();
};


DB.setTypes = function setTypes(object, types) {
	var prop = {};

	function st(f, t) {
		Object.keys(f).forEach(function (type) {
			t[type] = this[type];
		}, f);
	}

	if (object.prototype.types) {
		st(object.prototype.types, prop);
	}
	st(types, prop);
	object.prototype.types = prop;
};

/**
 * Get a singleton DB instance. If configuration is not specified,
 * it will be loaded from the DB configuration file using the same
 * group as the name.
 *
 *     // Load the default DB
 *     $db = DB::instance();
 *
 *     // Create a custom configured instance
 *     $db = DB::instance('custom', $config);
 *
 * @param   string   instance name
 * @param   array    configuration parameters
 * @return  DB
 */
DB.init = function init(core, config) {
	config = config || {};
	if (!config['type']) {
		throw new Exception(0, 'DB type not defined in configuration.');
	}
	var module = 'gc-db-' + config['type'].toLowerCase();
	try {
		module = require(module);
	} catch (e) {
		throw new Exception(0, 'GC database module :module isn\'t installed.', {module: module});
	}
	// Create the DB connection instance
	return new module(core, config);
};

DB.setTypes(DB, {
	// SQL-92
	'bit': {'type': 'string', 'exact': true},
	'bit varying': {'type': 'string'},
	'char': {'type': 'string', 'exact': true},
	'char varying': {'type': 'string'},
	'character': {'type': 'string', 'exact': true},
	'character varying': {'type': 'string'},
	'date': {'type': 'string'},
	'dec': {'type': 'float', 'exact': true},
	'decimal': {'type': 'float', 'exact': true},
	'double precision': {'type': 'float'},
	'float': {'type': 'float'},
	'int': {'type': 'int', 'min': '-2147483648', 'max': '2147483647'},
	'integer': {'type': 'int', 'min': '-2147483648', 'max': '2147483647'},
	'interval': {'type': 'string'},
	'national char': {'type': 'string', 'exact': true},
	'national char varying': {'type': 'string'},
	'national character': {'type': 'string', 'exact': true},
	'national character varying': {'type': 'string'},
	'nchar': {'type': 'string', 'exact': true},
	'nchar varying': {'type': 'string'},
	'numeric': {'type': 'float', 'exact': true},
	'real': {'type': 'float'},
	'smallint': {'type': 'int', 'min': '-32768', 'max': '32767'},
	'time': {'type': 'string'},
	'time with time zone': {'type': 'string'},
	'timestamp': {'type': 'string'},
	'timestamp with time zone': {'type': 'string'},
	'varchar': {'type': 'string'},

	// SQL:1999
	'binary large object': {'type': 'string', 'binary': true},
	'blob': {'type': 'string', 'binary': true},
	'boolean': {'type': 'bool'},
	'char large object': {'type': 'string'},
	'character large object': {'type': 'string'},
	'clob': {'type': 'string'},
	'national character large object': {'type': 'string'},
	'nchar large object': {'type': 'string'},
	'nclob': {'type': 'string'},
	'time without time zone': {'type': 'string'},
	'timestamp without time zone': {'type': 'string'},

	// SQL:2003
	'bigint': {'type': 'int', 'min': '-9223372036854775808', 'max': '9223372036854775807'},

	// SQL:2008
	'binary': {'type': 'string', 'binary': true, 'exact': true},
	'binary varying': {'type': 'string', 'binary': true},
	'varbinary': {'type': 'string', 'binary': true}
});

DB.Result = require('./result.js');
DB.Exception = require('./exception.js');
DB.Select = require('./select.js');
;