"use strict";
module.exports = Expression;

/**
 * Sets the expression string.
 *
 *   var expression = core.db.Expr('COUNT(users.id)');
 *
 * @constructor
 * @param db
 * @param {string} value Raw SQL expression string
 * @param {object} params Unquoted parameter values
 */
function Expression(db, value, params) {
	Object.defineProperties(this, { 
		db   : { value : db, writable : false},
		value : { value : value, writable : false}
	});
	this._params = {};
	if (params instanceof Object) {
		this.parameters(params);
	}
}


/**
 * Set the value of a parameter.
 *
 * @param   string  $param  parameter key to replace
 * @param   mixed   $value  value to use
 * @return  $this
 */
Expression.prototype.param = function(param, value) {
	this._params[param] = value;
	return this;
};

/**
 * Add multiple parameter values.
 *
 * @param   array   $params list of parameter values
 * @return  $this
 */
Expression.prototype.parameters = function(params) {
	for (var key in params) {
		this.param(key, params[key]);
	}
	return this;
};

/**
 * Return the value of the expression as a string.
 *
 *     echo $expression;
 *
 * @return  string
 * @uses    Database_Expression::value
 */
Expression.prototype.toString = function() {
	return this.compile();
};

/**
 * Compile the SQL expression and return it. Replaces any parameters with
 * their given values.
 *
 * @param   mixed    Database instance or name of instance
 * @return  string
 */
Expression.prototype.compile = function() {
	var keys = Object.keys(this._params),
		value = this.value;
	if ( keys.length ) {
		// Replace the values in the expression
		value = value.replace(new RegExp(':(' + keys.join('|') + ')', 'g'), function(param) {
			// Quote all of the parameter values
			return this.db.quote(this._params[param]);
		}.bind(this));
	}
	return value;
};