"use strict";
module.exports = Insert;
var Builder = require('./builder.js'),
	Select = require('./select.js'),
	Exception = require('./exception.js');

/**
 * Database query builder for INSERT statements. See [Query Builder](/database/query/builder) for usage and examples.
 *
 * @implements {Builder}
 * @param {DB} db
 * @param {string} table
 * @param {array} columns
 * @constructor
 */
function Insert(db, table, columns) {
	Builder.call(this, db, db.INSERT);
	this._table = table || null;
	this._columns = columns || [];
}
Insert.inherits(Builder);

/**
 * Sets the table to insert into.
 *
 * @param   mixed  table name or array($table, $alias) or object
 * @return  $this
 */
Insert.prototype.table = function table(table) {
	this._table = table;
	return this;
};

/**
 * Set the columns that will be inserted.
 *
 * @param   array  column names
 * @return  $this
 */
Insert.prototype.columns = function columns(columns) {
	this._columns = columns;
	return this;
};

/**
 * Adds or overwrites values. Multiple value sets can be added.
 *
 * @param   array   values list
 * @param   ...
 * @return  $this
 */
Insert.prototype.values = function values(values) {
	if (!Array.isArray(this._values)) {
		throw new Exception(0, 'INSERT INTO ... SELECT statements cannot be combined with INSERT INTO ... VALUES');
	}
	console.log(arguments);
	// Get all of the passed values
	this._values.push.apply(this._values, arguments);
	return this;
};

/**
 * Use a sub-query to for the inserted values.
 *
 * @param   object  Database_Query of SELECT type
 * @return  $this
 */
Insert.prototype.select = function select(query) {
	if ((!query instanceof Select) || (query.type !== this.db.SELECT)) {
		throw new Exception('Only SELECT queries can be combined with INSERT queries');
	}

	this._values = query;

	return this;
};

/**
 * Compile the SQL query and return it.
 *
 * @param   object  Database instance
 * @return  string
 */
Insert.prototype.compile = function compile() {
	// Start an insertion query
	var query = 'INSERT INTO ' + this.db.quoteTable(this._table);

	// Add the column names
	query += ' (' + this._columns.map(this.db.quoteColumn, this.db).join(', ') + ') ';
	//query += ' ('.implode(', ', array_map(array($db, 'quote_column'), )).') ';
	if (Array.isArray(this._values)) {
		// Callback for quoting values
		//$quote = array($db, 'quote');

		var groups = [];
		for (var i = 0, l = this._values.length; i < l; ++i) {
			var group = this._values[i];
			var g = [];
			for (var offset in group) {
				if (group.hasOwnProperty(offset)) {
					var value = group[offset];
					if ((typeof value != 'string') || (!(value in this._params))) {
						// Quote the value, it is not a parameter
						g.push(this.db.quote(value));
					} else {
						g.push(value);
					}
				}
			}
			groups.push('(' + g.join(', ') + ')');
		}
		// Add the values
		query += 'VALUES ' + groups.join(', ');
	} else {
		// Add the sub-query
		query += this._values.compile();
	}
	this._sql = query;

	return Builder.prototype.compile.call(this);
};

Insert.prototype.reset = function reset() {
	this._table = null;
	this._columns = [];
	this._values = [];
	return Builder.prototype.reset.call(this);
};
