"use strict";
module.exports = Join;
var Builder = require('./builder.js'),
	Exception = require('./exception.js');

/**
 * Creates a new JOIN statement for a table. Optionally, the type of JOIN
 * can be specified as the second parameter.
 *
 * @param   mixed   column name or array($column, $alias) or object
 * @param   string  type of JOIN: INNER, RIGHT, LEFT, etc
 * @return  void
 */
function Join(db, table, type) {
	Builder.call(this, db, 0);
	this._table = table;
	if (type) {
		this._type = type;
	}
}
Join.inherits(Builder);



/**
 * Adds a new condition for joining.
 *
 * @param   mixed   column name or array($column, $alias) or object
 * @param   string  logic operator
 * @param   mixed   column name or array($column, $alias) or object
 * @return  $this
 */
Join.prototype.on = function on(c1, op, c2) {
	if (this._using.length > 0) {
		throw new Exception(0, 'JOIN ... ON ... cannot be combined with JOIN ... USING ...');
	}
	this._on.push([c1, op, c2]);

	return this;
};

/**
 * Adds a new condition for joining.
 *
 * @param   string  column name
 * @param   ...
 * @return  $this
 */
Join.prototype.using = function using() {
	if (this._on.length > 0) {
		throw new Exception(0, 'JOIN ... ON ... cannot be combined with JOIN ... USING ...');
	}

	this._using.push.apply(this._using, arguments);

	return this;
};

/**
 * Compile the SQL partial for a JOIN statement and return it.
 *
 * @param   object  Database instance
 * @return  string
 */
Join.prototype.compile = function compile(db) {
	var sql;
	if (this._type) {
		sql = this._type.toUpperCase() + ' JOIN';
	} else {
		sql = 'JOIN';
	}

	// Quote the table name that is being joined
	sql += ' ' + this.db.quoteTable(this._table);

	if (this._using.length > 0) {
		// Quote and concat the columns
		sql += ' USING (' + this._using.map(this.db.quoteColumn, this.db).join(', ') + ')';
	} else {
		var conditions = [];
		this._on.forEach(function(condition) {
			// Split the condition
			var op = condition[1];

			if (op) {
				// Make the operator uppercase and spaced
				op = ' ' + op.toUpperCase();
			}

			// Quote each of the columns used for the condition
			conditions.push(this.db.quoteColumn(condition[0]) + op + ' ' + this.db.quoteColumn(condition[2]));
		}, this);

		// Concat the conditions "... AND ..."
		sql += ' ON (' + conditions.join(' AND ') + ')';
	}

	return sql;
};

/**
 *
 *  @return {Join}
 */
Join.prototype.reset = function reset() {
	this._type = null;
	this._table = null;
	this._on = [];
	this._using = [];
	Builder.prototype.reset.call(this);
	return this;
};