"use strict";
module.exports = Query;

/**
 * Query
 *
 * @param {DB} db
 * @param {number} type
 * @param {string} sql
 */
function Query(db, type, sql) {
	Object.defineProperties(this, {
		db   : { value : db, writable : false},
		type : { value : type, writable : false}
	});
	this._sql = sql;
	this.reset();
}
/**
 * DB instance
 * @type DB
 */
Query.prototype.db = null;

/**
 * To object converter
 * @type {function}
 */
Query.prototype._toObject = false;
/**
 * Compile and return SQL
 * @return {String}
 */
Query.prototype.toString = function toString() {
	return this.compile();
};
//
// // Cache lifetime
// _lifetime: null,
// _forceExecute: false,
//
//
// / **
// * Enables the query to be cached for a specified amount of time.
// *
// * @param   integer  number of seconds to cache, 0 deletes it from the cache
// * @param   boolean  whether or not to execute the query during a cache hit
// * @return  $this
// * @uses    Kohana::$cache_life
// * /
// cached: function cached(lifetime, force) {
// 	if (lifetime == null) {
// 		// Use the global setting
// 		lifetime = 600;
// 	}
//
// 	this._forceExecute = force || false;
// 	this._lifetime = lifetime;
//
// 	return this;
// },
//

/**
 * Returns row as objects
 *
 * @param   {function} func
 * @return  {Query}
 */
Query.prototype.toObject = function toObject(func) {
	this._toObject = func;
	return this;
};

/**
 * Set the value of a parameter in the query.
 *
 * @param   {string} param   parameter key to replace
 * @param   value    value to use
 * @return  {Query}
 */
Query.prototype.param = function param(param, value) {
	// Add or overload a new parameter
	this._params[param] = value;
	return this;
};

/**
 * Add multiple parameters to the query.
 *
 * @param   {object} params  list of parameters
 * @return  {Query}
 */
Query.prototype.parameters = function parameters(params) {
	// Merge the new parameters in
	for (var key in params) {
		this.param(key, params[key]);
	}
	return this;
};

/**
 * Compile the SQL query and return it. Replaces any parameters with their
 * given values.
 *
 * @return  string
 */
Query.prototype.compile = function compile() {
	// Import the SQL locally
	var sql = new String(this._sql),
		keys = Object.keys(this._params);
	if (keys.length) {
		// Replace the values in the expression
		sql = sql.replace(new RegExp(':(' + keys.join('|') + ')', 'g'), function (param, key) {
			// Quote all of the parameter values
			return this.db.quote(this._params[key]);
		}.bind(this));
	}

	return sql.toString();
};

/**
 * Execute the current query on the given database.
 *
 * @param {function} callback
 * @return {Query}
 */
Query.prototype.execute = function execute(callback) {
	// Compile the SQL query
	var sql = this.compile();
	if (this.type !== this.db.SELECT) {
		this.db.query(this.type, sql, callback);
	} else {
		var cache = this.db.cache.get(sql);
		if (!cache) {
			cache = this.db.cache.create(sql);
			// Execute the query
			this.db.query(this.type, sql, cache.doneF.bind(cache));
		}
		cache.add(callback);
	}
	return this;
};

Query.prototype.executeSync = function executeSync(callback) {
	// Compile the SQL query
	var sql = this.compile();
	// Execute the query
	return this.db.querySync(this.type, sql);
};
/**
 * Reset parameters
 * @return {Query}
 */
Query.prototype.reset = function reset() {
	this._params = {};
	return this;
};