"use strict";
module.exports = Select;

var Builder = require('./builder.js'),
	Where = require('./where.js'),
	Join = require('./join.js'),
	Exception = require('./exception.js');

/**
 * Sets the initial columns to select from.
 *
 * @implements Where
 * @param   array  column list
 * @return  void
 */
function Select(db, columns) {
	Where.call(this, db, db.SELECT);
	this.selectArray(columns || []);
}

Select.inherits(Where);

/**
 * Enables or disables selecting only unique columns using "SELECT DISTINCT"
 *
 * @param   {boolean} value  enable or disable distinct columns
 * @return  {Select}
 */
Select.prototype.distinct = function distinct(value) {
	this._distinct = value ? true : false;
	return this;
};

/**
 * Choose the columns to select from.
 *
 * @param  {string|Array|Expression} column Column name
 * @return  {Select}
 */
Select.prototype.select = function select(columnt /*,...*/) {
	return this.selectArray(arguments);
};

/**
 * Choose the columns to select from, using an array.
 *
 * @param   {Array} columns List of column names or aliases
 * @return  {Select}
 */
Select.prototype.selectArray = function selectArray(columns) {
	Array.prototype.forEach.call(columns, function (arg) {
		this._select.push(arg);
	}, this);
	return this;
};

/**
 * Choose the tables to select "FROM ..."
 *
 * @param {string|Array|Expression} table Table name or array($table, $alias) or object
 * @return  {Select}
 */
Select.prototype.from = function from(from/*,...*/) {
	Array.prototype.forEach.call(arguments, function (arg) {
		this._from.push(arg);
	}, this);
	return this;
};

/**
 * Adds addition tables to "JOIN ...".
 *
 * @param   {string|Array|Expression} table Table name or array($column, $alias) or object
 * @param   {string} type  Join type (LEFT, RIGHT, INNER, etc)
 * @return  {Select}
 */
Select.prototype.join = function join(table, type) {
	this._lastJoin = new Join(this.db, table, type);
	this._join.push(this._lastJoin);
	return this;
};

/**
 * Adds "ON ..." conditions for the last created JOIN statement.
 *
 * @param   {string|Array|Expression} c1 Column name or array($column, $alias) or object
 * @param   {string} op  Logic operator
 * @param   {string|Array|Expression} c2 Column name or array($column, $alias) or object
 * @return  {Select}
 */
Select.prototype.on = function on(c1, op, c2) {
	this._lastJoin.on(c1, op, c2);
	return this;
};

/**
 * Adds "USING ..." conditions for the last created JOIN statement.
 *
 * @param {string[]} column Column name
 * @return  {Select}
 */
Select.prototype.using = function using(column /*,...*/) {
	this._lastJoin.using.apply(this._lastJoin, arguments);
	return this;
};

/**
 * Creates a "GROUP BY ..." filter.
 *
 * @param   {string} column  Column name or array($column, $alias) or object
 * @return  {Select}
 */
Select.prototype.groupBy = function groupBy(column /*, ...*/) {
	Array.prototype.forEach.call(arguments, function (arg) {
		this._groupBy.push(arg);
	}, this);
	return this;
};

/**
 * Alias for `andHaving`
 *
 * @param   {string} column Column name or array($column, $alias) or object
 * @param   {string} op  logic operator
 * @param   {string|Array|Expression} value Column value
 * @return  {Select}
 */
Select.prototype.having = function having(column, op, value) {
	return this.andHaving(column, op, value);
};

/**
 * Creates a new "AND HAVING" condition for the query.
 *
 * @param   {string} column Column name or array($column, $alias) or object
 * @param   {string} op  logic operator
 * @param   {string|Array|Expression} value Column value
 * @return  {Select}
 */
Select.prototype.andHaving = function andHaving(column, op, value) {
	this._having.push({
		'AND': [column, op, value]
	});
	return this;
};

/**
 * Creates a new "OR HAVING" condition for the query.
 *
 * @param   {string} column Column name or array($column, $alias) or object
 * @param   {string} op  logic operator
 * @param   {string|Array|Expression} value Column value
 * @return  {Select}
 */
Select.prototype.orHaving = function orHaving(column, op, value) {
	this._having.push({
		'OR': [column, op, value]
	});
	return this;
};

/**
 * Opens a new "AND HAVING (...)" grouping.
 *
 * @return  {Select}
 */
Select.prototype.andHavingOpen = function andHavingOpen() {
	this._having.push({
		'AND': '('
	});
	return this;
};

/**
 * Alias for `andHavingOpen`
 * @return {Select}
 */
Select.prototype.havingOpen = function havingOpen() {
	return this.andHavingOpen();
};

/**
 * Opens a new "OR HAVING (...)" grouping.
 *
 * @return  {Select}
 */
Select.prototype.orHavingOpen = function orHavingOpen() {
	this._having.push({
		'OR': '('
	});
	return this;
};

/**
 * Alias for andHavingClose
 * @return {Select}
 */
Select.prototype.havingClose = function havingClose() {
	this.andHavingClose();
	return this;
};

/**
 * Closes an open "AND HAVING (...)" grouping.
 *
 * @return  {Select}
 */
Select.prototype.andHavingClose = function andHavingClose() {
	this._having.push({
		'AND': ')'
	});
	return this;
};

/**
 * Closes an open "OR HAVING (...)" grouping.
 *
 * @return  {Select}
 */
Select.prototype.orHavingClose = function orHavingClose() {
	this._having.push({
		'OR': ')'
	});
	return this;
};

/**
 * Adds an other UNION clause.
 *
 * @param {string|Select} select if string, it must be the name of a table. Else
 *  must be an instance of {Select}
 * @param {boolean} all decides if it's an UNION or UNION ALL clause
 * @return {Select}
 */
Select.prototype.union = function union(select, all) {
	if (typeof all == 'undefined') {
		all = true;
	}
	if (typeof select == 'string') {
		select = new Select(this.db).from(select);
	}
	if (!select instanceof Select) {
		throw new Exception('first parameter must be a string or an instance of Select');
	}
	this._union.push({
		select: select,
		all: all
	});
	return this;
};

/**
 * Start returning results after "OFFSET ..."
 *
 * @param   {number} number   starting result number or NULL to reset
 * @return  {Select}
 */
Select.prototype.offset = function offset(number) {
	this._offset = number;
	return this;
};

/**
 * Compile the SQL query and return it.
 *
 * @param   object  Database instance
 * @return  string
 */
Select.prototype.compile = function compile() {
	// Start a selection query
	var query = 'SELECT ',
		db = this.db;

	if (this._distinct === true) {
		// Select only unique results
		query += 'DISTINCT ';
	}
	if (this._select.length == 0) {
		// Select all columns
		query += '*';
	} else {
		// Select all columns
		query += this._select.map(db.quoteColumn, db).unique().join(', ');
	}

	if (this._from.length != 0) {
		// Set tables to select from
		query += ' FROM ' + this._from.map(db.quoteTable, db).unique().join(', ');
	}

	if (this._join.length != 0) {
		// Add tables to join
		query += ' ' + this._compileJoin(this._join);
	}

	if (this._where.length != 0) {
		// Add selection conditions
		query += ' WHERE ' + this._compileСonditions(this._where);
	}

	if (this._groupBy != 0) {
		// Add grouping
		query += ' ' + this._compileGroupBy(this._groupBy);
	}

	if (this._having.length > 0) {
		// Add filtering conditions
		query += ' HAVING ' + this.$compileConditions(this._having);
	}

	if (this._orderBy.length > 0) {
		// Add sorting
		query += ' ' + this._compileOrderBy(this._orderBy);
	}

	if (this._limit !== null) {
		// Add limiting
		query += ' LIMIT ' + this._limit;
	}

	if (this._offset !== null) {
		// Add offsets
		query += ' OFFSET ' + this._offset;
	}

	if (this._union.length > 0) {
		this._union.forEach(function (u) {
			//foreach (this._union as $u) {
			query += ' UNION ';
			if (u['all'] === true) {
				query += 'ALL ';
			}
			query += u['select'].compile();
		}, this);
	}

	this._sql = query;

	return Where.prototype.compile.call(this);
};

Select.prototype.reset = function reset() {
	this._select = [];
	this._from = [];
	this._join = [];
	this._groupBy = [];
	this._having = [];
	this._union = [];
	this._distinct = false;
	this._offset = null;
	this._lastJoin = null;
	return Where.prototype.reset.call(this);
};