"use strict";
module.exports = Update
var Where = require('./where.js');

/**
 * Database query builder for UPDATE statements. See [Query Builder](/database/query/builder) for usage and examples.
 *
 * @implements {Where}
 * @param {DB} db
 * @param {string} table
 * @constructor
 */
function Update(db, table) {
	Where.call(this, db, db.UPDATE);
	if (table) {
		this._table = table;
	}
}

Update.inherits(Where);
 
/**
* Sets the table to update.
*
* @param   table String  table name or [table, alias] or object
* @return  this
*/
Update.prototype.table = function table(table) {
	this._table = table;
	return this;
};

/**
* Set the values to update with an associative array.
*
* @param   pairs   associative (column => value) list
* @return  $this
*/
Update.prototype.set = function set(pairs) {
	for (var column in pairs) {
		this.value(column, pairs[column]);
	}
	return this;
};

/**
* Set the value of a single column.
*
* @param   mixed  table name or array($table, $alias) or object
* @param   mixed  column value
* @return  $this
*/
Update.prototype.value = function value(column, value) {
	this._set.push([column, value]);
	return this;
};

/**
* Compile the SQL query and return it.
*
* @param   object  Database instance
* @return  string
*/
Update.prototype.compile = function compile() {
	// Start an update query
	var query = 'UPDATE ' + db.quoteTable(this._table);
	
	// Add the columns to update
	query += ' SET ' + this._compileSet(this._set);
	
	if (this._where.length > 0) {
		// Add selection conditions
		query += ' WHERE ' + this._compileСonditions(this._where);
	}
	if (this._orderBy.length > 0) {
		// Add sorting
		query += ' ' + this._compileOrderBy(this._orderBy);
	}
	if (this._limit !== null){
		// Add limiting
		query += ' LIMIT ' + this._limit;
	}
	this._sql = query;

	return Where.prototype.compile.call(this);
};

Update.prototype.reset = function reset() {
	this._table = null;
	this._set   = [];
	return Where.prototype.reset.call(this);
};