"use strict";
module.exports = Where;
var Builder = require('./builder.js');

/**
 * Where
 *
 * @implements {Builder}
 * @constructor
 */
function Where() {
	Builder.apply(this, arguments);
}
Where.inherits(Builder);

/**
 * Creates a new "AND WHERE" condition for the query.
 *
 * @param {string|Array|Expression} column Column name or array($column, $alias) or object
 * @param {string} op Logic operator
 * @param value Column value
 * @return  {Where}
 */
Where.prototype.where = function where(column, op, value) {
	return this.andWhere(column, op, value);
};

Where.prototype.andWhere = function (column, op, value) {
	this._where.push({
		'AND': [column, op, value]
	});
	return this;
};

/**
 * Creates a new "OR WHERE" condition for the query.
 *
 * @param   mixed   column name or array($column, $alias) or object
 * @param   string  logic operator
 * @param   mixed   column value
 * @return  $this
 */
Where.prototype.orWhere = function orWhere(column, op, value) {
	this._where.push({
		'OR': [column, op, value]
	});
	return this;
};

/**
 * Opens a new "AND WHERE (...)" grouping.
 *
 * @return  $this
 */
Where.prototype.whereOpen = function whereOpen() {
	return this.andWhereOpen();
};

Where.prototype.andWhereOpen = function andWhereOpen() {
	this._where.push({
		'AND': '('
	});
	return this;
};

/**
 * Opens a new "OR WHERE (...)" grouping.
 *
 * @return  $this
 */
Where.prototype.orWhereOpen = function orWhereOpen() {
	this._where.push({
		'OR': '('
	});
	return this;
};

/**
 * Closes an open "AND WHERE (...)" grouping.
 *
 * @return  $this
 */
Where.prototype.whereClose = function whereClose() {
	return this.andWhereClose();
};

Where.prototype.andWhereClose = function andWhereClose() {
	this._where.push({
		'AND': ')'
	});
	return this;
};

/**
 * Closes an open "OR WHERE (...)" grouping.
 *
 * @return  $this
 */
Where.prototype.orWhereClose = function orWhereClose() {
	this._where.push({
		'OR': ')'
	});
	return this;
};

/**
 * Applies sorting with "ORDER BY ..."
 *
 * @param   mixed   column name or array($column, $alias) or object
 * @param   string  direction of sorting
 * @return  $this
 */
Where.prototype.orderBy = function orderBy(column, direction) {
	this._orderBy.push([column, direction]);
	return this;
};

/**
 * Return up to "LIMIT ..." results
 *
 * @param   integer  maximum results to return or NULL to reset
 * @return  $this
 */
Where.prototype.limit = function limit(number) {
	this._limit = number;
	return this;
};

Where.prototype.reset = function reset() {
	this._where = [];
	this._orderBy = [];
	this._limit = null;
	Builder.prototype.reset.call(this);
	return this;
};

